{

=head1 NAME

Net::Google::Search

=head1 SYNOPSIS

 use Net::Google::Search;
 my $search = Net::Google::Search($service,%args);

=head1 DESCRIPTION

This package is used by I<Net::Google>. It should not be used on it's own.

=cut

package Net::Google::Search;
use strict;

use Carp;
use Exporter;

$Net::Google::Search::VERSION   = 0.1;
@Net::Google::Search::ISA       = qw (Exporter);
@Net::Google::Search::EXPORT    = qw ();
@Net::Google::Search::EXPORT_OK = qw ();

=head1 Class Methods

=head2 $pkg = Net::Google::Search->new($service,%args)

Where I<$service> is a valid I<GoogleSearchService> object.

Valid arguments are :

=over

=item B<key>

String. Google API key.

=item B<starts_at>

Int. First result number to display. Default is 0.

=item B<max_results>

Int. Number of results to return. Default is 10.

=item B<lr>

Array reference. Language restrictions.

=item B<ie>

String. Input encoding.

=item B<oe>

String. Output encoding.

=item B<safe>

Boolean.

=item B<filter>

Boolean.

=back

=cut

sub new {
  my $pkg = shift;
  
  my $self = {};
  bless $self,$pkg;

  if (! $self->init(@_)) {
    return undef;
  }

  return $self;
}

sub init {
  my $self    = shift;
  my $service = shift;
  my $args    = shift;

  if (ref($service) ne "GoogleSearchService") {
    carp "Unknown service";
    return 0;
  }

  $self->{'_service'}  = $service;

  if (! $args->{'key'}) {
    carp "You must define a key";
    return 0;
  }

  $self->key($args->{'key'});

  $self->{'_query'}       = [];
  $self->{'_lr'}          = [];
  $self->{'_ie'}          = "";
  $self->{'_oe'}          = "";
  $self->{'_safe'}        = 0;
  $self->{'_filter'}      = 0;
  $self->{'_starts_at'}   = 0;
  $self->{'_max_results'} = 10;

  $self->starts_at(($args->{'starts_at'} || 0));
  $self->max_results(($args->{'max_results'}) || 10);

  if (ref($args->{lr}) eq "ARRAY") {
    defined($self->lr(@{$args->{'lr'}})) || return 0;
  }

  if (my $ie = $args->{'ie'}) {
    defined($self->ie($ie)) || return 0;
  }

  if (my $oe = $args->{'oe'}) {
    defined($self->oe($oe)) || return 0;
  }

  if (defined($args->{'filter'})) {
    defined($self->filter($args->{'filter'})) || return 0;
  }

  if (defined($args->{'safe'})) {
    defined($self->safe($args->{'safe'})) || return 0;
  }

  if (defined($args->{'starts_at'})) {
    defined($self->starts_at($args->{'starts_at'})) || return 0;
  }

  if (defined($args->{'max_results'})) {
    defined($self->max_results($args->{'max_results'})) || return 0;
  }

  return 1;
}

=head2 $pkg->key()

Returns a string.

Returns undef if there was an error.

=cut

sub key {
  my $self = shift;
  my $key  = shift;

  if (defined($key)) {
    $self->{'_key'} = $key;
  }

  return $self->{'_key'};
}

=head2 $pkg->query(@data)

Adds the contents of I<@data> to the list of words to query for. If the first item in I<@data> is blank, or undef, then the current list will be entirely replace by the one being passed to the method.

Returns a string of words separated by white space.

Returns undef if there was an error.

=cut

sub query {
  my $self = shift;
  my @data = @_;

  if ((scalar(@data) > 1) && ($data[0] eq "")) {
    $self->{'_query'} = [];
  }

  push @{$self->{'_query'}}, @data;
  return join(" ",@{$self->{'_query'}});
}

=head2 $pkg->starts_at($at)

Default is 0.

Returns an int.

Returns undef if there was an error.

=cut

sub starts_at {
  my $self = shift;
  my $at   = shift;

  if (defined($at)) {
    $self->{'_starts_at'} = $at;
  }

  return $self->{'_starts_at'};
}

=head2 $pkg->max_results($max)

The default set by Google is 10 results. However, if you pass a number greater than 10 the I<results> method will make multiple calls to Google API.

Returns an int.

Returns undef if there was an error.

=cut

sub max_results {
  my $self = shift;
  my $max  = shift;

  if (defined($max)) {

    if (int($max) < 1) {
      carp "'$max' must be a int greater than 0";
      $max = 1;
    }

    $self->{'_max_results'} = $max;
  }

  return $self->{'_max_results'};
}

=head2 $pkg->restrict()

B<This method still needs to be implemented.>

Returns undef if there was an error.

=cut

sub restrict {
  my $self = shift;
}

=head2 $pkg->filter($bool)

Returns true or false.

Returns undef if there was an error.

=cut

sub filter {
  my $self = shift;
  my $bool = shift;

  if (defined($bool)) {
    $self->{'_filter'} = ($bool) ? "true" : "false";;
  }

  return $self->{'_filter'};
}

=head2 $pkg->safe($bool)

Returns true or false.

Returns undef if there was an error.

=cut

sub safe {
  my $self = shift;
  my $bool = shift;

  if (defined($bool)) {
    $self->{'_safe'} = ($bool) ? "true" : "false";;
  }

  return $self->{'_safe'};
}

=head2 $pkg->lr(@lang)

Language restriction.

Returns a list of language codes separated by a "|"

Returns undef if there was an error.

=cut

sub lr {
  my $self = shift;
  my @lang = @_;

  if (@lang) {

    if ((scalar(@lang) == 1) && ($lang[0] eq "")) {
      $self->{'_lr'} = [];
      shift @lang;
    } 

    foreach my $l (@lang) {

      if ($l =~ /^(ar|zh-CN|zh-TW|cs|da|nl|en|et|fi|fr|de|el|iw|hu|is|it|ja|ko|lv|lt|no|pt|pl|ro|ru|es|sv|tr)$/) {
	push @{$self->{'_lr'}}, "lang_".$1;
      }
      
      else {
	carp "Unknown language key : '$l'";
      }
    }
    
  }

  return join("|",@{$self->{'_lr'}});
}

=head2 $pkg->ie($type)

Returns a string.

Returns undef if there was an error.

=cut

sub ie {
  my $self = shift;
  my $type = shift;

  if (defined($type)) {
    $self->{'_ie'} = ($type) ? &_encoding($type) : undef;
  }

  return $self->{'_ie'};
}

=head2 $pkg->oe ($type)

Returns a string.

Returns undef if there was an error.

=cut

sub oe {
  my $self = shift;
  my $type = shift;
  
  if (defined($type)) {
    $self->{'_oe'} = ($type) ? &_encoding($type) : undef;
  }
  
  return $self->{'_oe'};
}

=head2 $pkg->results()

Returns an array ref of I<Result> objects.

=cut

sub results {
  my $self = shift;

  my $start_at = $self->starts_at();
  my $to_fetch = $self->max_results();
  my $results  = [];

  while ($to_fetch > 0) {
    my $count = ($to_fetch > 10) ? 10 : $to_fetch;
    push @{$results}, @{$self->_results($start_at,$count)};

    $start_at += 10;
    $to_fetch -= 10;
  }

  return $results;
}

=head1 Private Methods

=head2 $pkg->_results($first,$count)

=cut

sub _results {
  my $self  = shift;
  my $first = shift;
  my $count = shift;

  my $result = $self->{'_service'}->doGoogleSearch(
						   $self->key(),
						   $self->query(),
						   $first,
						   $count,
						   SOAP::Data->type(boolean =>$self->filter()),
						   "", #restrict
						   SOAP::Data->type(boolean =>$self->safe()),
						   $self->lr(),
						   $self->ie(),
						   $self->oe(),
						  );

  if(ref($result->{resultElements}) eq "ARRAY") {
    map { $_ = Result->new($_); } @{$result->{resultElements}};
    return $result->{resultElements};
  }

  return [];
}

=head1 Private Functions

=head2 &_encoding($type)

=cut

sub _encoding {
  my $type = shift;

  if ($type =~ /^(arabic|gb|big5|latin1|latin2|latin3|latin4|latin5|latin6|greek|hebrew|sjis|euc-jp|euc-kr|cyrillic|utf8)$/) {
    return $1;
  }
  
  else {
    carp "Unknown encoding type : '$type'";
    return undef;
  }
}

=head1 Result Methods

=cut

package Result;
use vars qw ($AUTOLOAD);

sub new {
  my $pkg = shift;
  my $res = shift;
  return bless $res,$pkg;
}

=head2 $result->title()

Returns a string.

=head2 $result->URL()

Returns a string.

=head2 $result->snippet()

Returns a string, formatted in HTML.

=head2 $result->cachedSize()

Returns a string.

=head2 $result->directoryTitle()

Returns a string.

=head2 $result->summary()

Returns a string.

=head2 $result->hostName()

Returns a string.

=head2 $result->directoryCategory()

Returns a hash reference.

=cut

sub AUTOLOAD {
  my $self = shift;
  $AUTOLOAD =~ s/.*:://;
  return $self->{$AUTOLOAD};
}

=head1 VERSION

0.1

=head1 DATE

$date$

=head1 AUTHOR

Aaron Straup Cope

=head1 TO DO

=over

=item 

Add hooks to manage boolean searches and speacial query strings.

=item

Finish I<restrict> method.

=back

=head1 SEE ALSO

L<Net::Google>

=head1 LICENSE

Copyright (c) 2002, Aaron Straup Cope. All Rights Reserved.

This is free software, you may use it and distribute it under the same terms as Perl itself.

=cut

return 1;

}
